<?php
// +--------------------------------------------------------------------+
// | PEAR, the PHP Extension and Application Repository                 |
// +--------------------------------------------------------------------+
// | Copyright (c) 1997-2004 The PHP Group                              |
// +--------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,     |
// | that is bundled with this package in the file LICENSE, and is      |
// | available through the world-wide-web at the following url:         |
// | http://www.php.net/license/3_0.txt.                                |
// | If you did not receive a copy of the PHP license and are unable to |
// | obtain it through the world-wide-web, please send a note to        |
// | license@php.net so we can mail you a copy immediately.             |
// +--------------------------------------------------------------------+
// | Author: Alexandre Quessy <alex@sourcelibre.com>                   |
// +--------------------------------------------------------------------+

require_once 'PEAR.php';
require_once 'Net/Server.php';
require_once 'Net/Socket.php';
require_once 'Audio/PureData/NetReceive.php';
require_once 'Audio/PureData/NetSend.php';
require_once 'Audio/PureData/ServerHandler.php';
/**
 * Object for two ways communication with Pure Data. 
 * Creates a socket server and client.
 */
class Audio_PureData_NetReceive {
    var $_verbose = FALSE;
    /**
     * @var object Audio_PureData_NetSend
     */
    var $_netsend;
    /**
     * @var object Audio_PureData_ServerHandler
     */
    var $_server_handler;
    /**
     * @var object Net_Server
     */
    var $_server;
    /**
     * Creates the TCP server and server handler. This receives the data from PD.
     * 
     * @param string $type Either 'fork' or 'sequential'
     * @param string $host
     * @param int $port
    */
    function Audio_PureData_NetReceive($host, $receive_port, $send_port) {
        $this->setNetSend($host, $send_port);
        $this->setNetReceive($host, $receive_port);
    }
    /**
     * Creates the TCP server and server handler. This receives the data from PD.
     * 
     * @param string $type Either 'fork' or 'sequential'
     * @param string $host
     * @param int $port
     */
    function setNetReceive($host, $port, $type = 'sequential') {
        $this->_server  = &Net_Server::create($type, $host, (int) $port); 
        $this->_server_handler = &new Audio_PureData_ServerHandler();
        if (isset($this->_netsend)) {
            $this->_server_handler->setPdSend(&$this->_netsend);
        }
        $this->_server->setCallbackObject($this->_server_handler);
    }
    
    /**
     * Creates the TCP client socket. This sends the data to PD.
     * 
     * @param boolean $udp Can only be FALSE for now
     * @param string $host
     * @param int $port
     */
    function setNetSend($host, $port, $udp = FALSE) {
        $this->netsend =& new Audio_PureData_NetSend($host, $port, $udp);
        if (isset($this->_server_handler)) {
            $this->_server_handler->setPdSend(&$this->_netsend);
        }
    }
    /**
     * @param boolean $mode Sets the debug mode on or off
     */
    function setVerbose($mode) {
        $this->_verbose = $mode;
    }
    /**
     * Starts the server and client. 
     * 
     * You can now use your [phpext] PD object. 
     * @todo Fix the error handler
     */
    function start() {
        $this->_server->start();
    }
    /**
     * Stops the server and client. Ends this script.
     */
    function stop() {
        $this->error("Closing the server now. Ciao !"); 
        $this->_send_socket->disconnect();
        $this->_server->shutDown();
    }
    /**
     * Send a message to the outlet of the [phpext] PD object. 
     * 
     * The argument can be an array or a string. It will be converted to a PD list or atom. 
     * @param array|string|float $msg
     */
    function send($msg) {
        $this->_netsend->send($msg); 
    }
    
    /**
     * Displays an error to the PD console.
     * 
     * @param string $msg
     */ 
    function error($msg) {
        @$this->_netsend->send('PHP error' . $msg);
    }
    /** 
     * Returns the Server Handler
     * Needed to add callback selectors
     * @return object Audio_PureData_ServerHandler Provides the method addSelector and such.
     */
    function &getServerHandler() {
        return $this->_server_handler;
    }
    /** 
     * Returns the NetSend object.
     * Needed to send data to PHP
     * @return object Audio_PureData_NetSend
    */
    function &getNetSend() {
        return $this->_netsend;
    }
    /** @see Audio_PureData_ServerHandler::addSelector() */
    function addSelector($selector, &$callback) {
        $this->_server_handler->addSelector($selector, &$callback);
    }
    /** @see Audio_PureData_ServerHandler::setSingleHandler() */
    function setSingleHandler(&$callback) {
        $this->_server_handler->setSingleHandler(&$callback);
    }
}

/*
 * Local Variables:
 * mode: php
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 */
?>
