<?php
// +--------------------------------------------------------------------+
// | PEAR, the PHP Extension and Application Repository                 |
// +--------------------------------------------------------------------+
// | Copyright (c) 1997-2004 The PHP Group                              |
// +--------------------------------------------------------------------+
// | This source file is subject to version 3.0 of the PHP license,     |
// | that is bundled with this package in the file LICENSE, and is      |
// | available through the world-wide-web at the following url:         |
// | http://www.php.net/license/3_0.txt.                                |
// | If you did not receive a copy of the PHP license and are unable to |
// | obtain it through the world-wide-web, please send a note to        |
// | license@php.net so we can mail you a copy immediately.             |
// +--------------------------------------------------------------------+
// | Author: Alexandre Quessy <alex@sourcelibre.com>                   |
// +--------------------------------------------------------------------+
/**
 * I hope this software helps you to express your creativity.
 * Please share your joy and tell me about it.
 */
require_once 'PEAR.php';
require_once 'Net/Socket.php';
/** 
 * The FUDI protocols parses the decimal to a pretty low precision.
 */
define('AUDIO_PUREDATA_NUMDECIMALS', 6);

/**
 * One way communication with Pure Data. From the web server to [netreceive].
 * 
 * Creates a client.
 */
class Audio_PureData_NetSend {
    /**
     * @var boolean  
     */
    var $_verbose = FALSE;
    /**
     * @var object Net_Socket
     */
    var $_send_socket;
    /**
     * Creates the TCP client socket. This sends the data to PD.
     * 
     * @param string $type Either 'fork' or 'sequential'
     * @param string $host
     * @param int $port
     * @param boolean $udp Defaults to TCP. No UDP support for now. 
     */
    function Audio_PureData_NetSend($host, $port, $udp = FALSE) {
        $this->_send_socket = new Net_Socket();
        $s_status = $this->_send_socket->connect($host, (int) $port, true); 
        // host, port, persistent, timeout
        if (PEAR::isError($s_status)) {
            echo "Impossible to send messages to PD : ";
            echo $s_status->getMessage() . "\n";
        } else {
            if ($this->_verbose) {
                echo("PHP sends " . $list . ". \n");
            }
            //echo "Connexion OK to Pure Data on port $port to $host. \n";
        }
    }
    /**
     * @param boolean $mode Sets the debug mode on or off
     */
    function setVerbose($mode = TRUE) {
        $this->_verbose = $mode;
    }
    /**
     * Sends a message to PureData's [netreceive]. 
     * 
     * The argument can be an array or a string. It will be converted to a PD list or atom. 
     * @param array|string|float $msg
     * @todo Remove forbidden characters.
     */
    function send($msg) {
        $list = "";
        if (!is_array($msg)) {
            $array = array($msg);
        } else {
            $array = $msg;
        }
        foreach ($array as $val) {
            if (is_numeric($val)) {
                $val = number_format((float) $val, AUDIO_PUREDATA_NUMDECIMALS); 
            }
            $list .= $val . " ";
        }
        
        $list .= ';';
        if ($this->_verbose) {
            print ("PHP sends " . $list . "\n");
        }
        $this->_send_socket->writeLine($list); 
    }
    /** 
     * Sends an error message to Pure Data 
     * @param $msg string
     */
    function error($msg) {
        $this->send("PHP_error " . $msg);
    }
}

/*
 * Local Variables:
 * mode: php
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 */
?>
