<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
require_once 'Net/Server/Handler.php';

/**
 * TCP server handler. This class can be overriden. No need to, you can just register callbacks.
 * 
 * Handles the events onReceiveData, onConnect and onClose. 
 * Calls the methods of the PhpExt object corresponding to 
 * the selectors of the Pure Data lists it receives. 
 * 
 * You must either use the single handler method, or use one function handler
 * for every selector.
 * 
 * In both cases, the first argument to the callback will be a reference to this 
 * Audio_PureData_ServerHandler object. The second argument will be an array or a 
 * string containing the data that is received from Pure Data.
 */ 
class Audio_PureData_ServerHandler extends Net_Server_Handler {
   /**
    * @var array Key = Pure Data selector (a string). value = callback to a PHP function.
    */
   var $_handlers;
   /** 
    * @var object Audio_PureData_NetSend 
    */
   var $net_send;
   /** 
    * @var callback Callback function for any message.  
    */
    var $single_handler;
   /**
    * This is where the translation from FUDI to PHP callbacks is operated. 
    * @param    integer $clientId
    * @param    string  $data
    */
    function onReceiveData($clientId = 0, $data = "") {
        $data = trim($data);
        $data = substr($data, 0, strlen($data) - 1);
        $arr = split(' ', $data);
        
        $func = $this->single_handler;
        if (is_callable($func)) {
            @call_user_func_array($func, array(&$this, $data));
        } else {
            // list to a selector and an array
            $selector = $arr[0];
            /* we do not need the first value, it is the selector */
            $args = $arr;
            array_shift($args);
        
            if (is_callable($this->_handlers[$selector])) {
                @call_user_func_array($this->_handlers[$selector], array(&$this, $args));
            } else {
                $this->net_send->error("No PHP method for " . $arr[0]);
            }
        }
    }
    
    /** You can override this method */
    function onConnect($clientId = 0) {
        if ($this->_verbose) {
            print ("PHP opens the connexion. PHP can now receive data from PD.\n");
        }
    }
    /** You can override this method */
    function onClose() {
        if ($this->_verbose) {
            print ("PHP closes the connexion. PHP can no longer receive data from PD.\n");
        }
        $this->net_send->disconnect();
    }
    /** 
     * Registers a PHP function as a callback
     * @var string $selector PD symbol
     * @var callback $callback Function name or an array containing a reference to 
     * an object and the name of the function to call. This function will receive an 
     * array containing the list of arguments. 
     */
    function addSelector($selector, &$callback) {
        if (is_callable($callback)) {
            $this->_handlers[$selector] = &$callback;
        } else {
            $this->net_send->error("This PHP function is not callable.");
        }
    }
    /** 
     * Registers a single PHP function as a callback for any message.
     * @var string $selector PD symbol
     * @var callback $callback Function name or an array containing a reference to 
     * an object and the name of the function to call. This function will receive an 
     * array containing the list of arguments. 
     */
    function setSingleHandler(&$callback) {
        if (is_callable($callback)) {
            $this->single_handler = &$callback;
        } else {
            $this->net_send->error("This PHP function is not callable.");
        }
    }
    
    /** 
     * @param $netsend object Audio_PureData_NetSend  
     */
    function setPdSend(&$netsend) {
        $this->net_send =& $netsend;
    }
    function send($data) {
        $this->net_send->send($data);
    }
}
?>
